-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011, 2012, 2013 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

local class	= require "engine.class"
local Dialog = require "engine.ui.Dialog"

class:bindHook("ToME:load", function(self, data)
	-- Check if the user needs to configure for the first time
	if ( (config.settings.display_fps_resting_div == nil) 
		or (config.settings.display_fps_running_div == nil) ) then
		-- Declare a global flag that other hooks can see
		faster_rre_firsttime_setup = 1
	end

	-- Initialize our divider settings
	-- Note: these settings are intended to be integrated directly into t-engine
	config.settings.display_fps_resting_div = config.settings.display_fps_resting_div or 1
	config.settings.display_fps_running_div = config.settings.display_fps_running_div or 1
	
	-- make sure these settings are sane
	if (config.settings.display_fps_resting_div < 1) then
		config.settings.display_fps_resting_div = 1
	elseif (config.settings.display_fps_resting_div > config.settings.display_fps) then
		config.settings.display_fps_resting_div = config.settings.display_fps
	end
	if (config.settings.display_fps_running_div < 1) then
		config.settings.display_fps_running_div = 1
	elseif (config.settings.display_fps_running_div > config.settings.display_fps) then
		config.settings.display_fps_running_div = config.settings.display_fps
	end
	
end)

class:bindHook("ToME:run", function(self, data)
	-- Pop up a "First Time Setup" dialog if necessary 
	-- Note: if the player escaped out of this dialog without modifying any
	--  settings (forcing a settings save), they will get this popup again!
	if (faster_rre_firsttime_setup) then
		local firstrun_text = "It appears this is your first time playing with"
			.. " the #{bold}#Faster Run/Rest/Explore#{normal}# addon."
			.. "\n\n#{bold}#Recommended settings:#{normal}#"
			.. "\n     #FIREBRICK##{bold}#Requested FPS#{normal}#: #GOLD#30"
			.. "\n     #FIREBRICK##{bold}#Resting FPS Divider#{normal}#: #GOLD#10"
			.. "\n     #FIREBRICK##{bold}#Run/Explore FPS Divider#{normal}#: #GOLD#5"
			.. "#WHITE#\n\nBefore playing you must configure these settings."
			.. " After this initial setup, they can be set by"
			.. " #{bold}#pressing escape and selecting video options#{normal}#."
			.. "\n\nThis addon works by dividing the requested FPS when resting"
			.. " or running. Setting these dividers to something greater than"
			.. " one can #{bold}#substantially#{normal}# reduce the amount of time"
			.. " required to finish resting/running on older computers."
			.. " Since resting may be #{bold}#much#{normal}# faster than normal,"
			.. " be extra careful before resting on an escort mission!"

		-- TODO: should probably scale this based on the current resolution...
		Dialog:listPopup("Faster Run/Rest/Explore - First Time Setup"
			, firstrun_text, {{name="Okay, let's set up this addon!", val=0}}, 700, 400
			, function(sel)
				local menu = require("engine.dialogs.VideoOptions").new()
				game:registerDialog(menu)
			  end
		)
		
		-- Blast this global flag away in case of an engine restart
		faster_rre_firsttime_setup = nil
	end

end)