-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011, 2012, 2013 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

local Dialog = require "engine.ui.Dialog"
local TreeList = require "engine.ui.TreeList"
local Textzone = require "engine.ui.Textzone"
local GetQuantity = require "engine.dialogs.GetQuantity"

local _M = loadPrevious(...)

local base_generateList = _M.generateList

function _M:generateList()
	-- Helper function for determining nice ratios for rest/run FPS
	-- Finds the factors of an integer, turns into list for Dialog:listPopup
	-- Based on: http://rosettacode.org/wiki/Factors_of_an_integer#Lua
	local function factors( n ) 
		local f = {} 
		
		for i = 1, n/2 do
			if n % i == 0 then 
				f[#f+1] = {name=tostring(i), val=tostring(i)}
			end
		end
		f[#f+1] = {name=tostring(n), val=tostring(n)}
 
		return f
	end
	local function fps_factor_explanation(verb)
		return "The " .. verb .. " FPS divider.  This number is a factor of the requested FPS."
				.. "\nDivides the requested FPS by this amount while " .. verb .. "."
				.. "\nIncreasing this divider may greatly reduce the time"
				.. " required to finish " .. verb .. " on older computers."
				.. "\n\nWhen set to #LIGHT_STEEL_BLUE##{bold}#1#WHITE##{normal}#"
				.. ", this setting will have no effect."
				.. "#WHITE#"
	end

	-- Generate the original list
	base_generateList(self)
	
	-- Fix the FPS list item
	local zone = Textzone.new{width=self.c_desc.w, height=self.c_desc.h, text=string.toTString"Request this display refresh rate.\nSet it lower to reduce CPU load, higher to increase interface responsiveness.\n\n#CYAN#Note: changing this setting will reset the current FPS dividers to #LIGHT_STEEL_BLUE##{bold}#1#WHITE##{normal}#.#WHITE#"}
	self.list[2] = { zone=zone, name=string.toTString"#FIREBRICK##{bold}#Requested FPS#WHITE##{normal}#", status=function(item)
			return tostring(config.settings.display_fps)
		end, fct=function(item)
			game:registerDialog(GetQuantity.new("Enter FPS", "From 5 to 60", config.settings.display_fps, 60, function(qty)
				qty = util.bound(qty, 5, 60)
				game:saveSettings("display_fps", ("display_fps = %d\n"):format(qty))
				game:saveSettings("display_fps_resting_div", "display_fps_resting_div = 1\n")
				game:saveSettings("display_fps_running_div", "display_fps_running_div = 1\n")
				config.settings.display_fps = qty
				config.settings.display_fps_resting_div = 1;
				config.settings.display_fps_running_div = 1;
				core.game.setFPS(qty)
				self.c_list:drawTree()
				
			end), 5)
		end,}
	
	-- Add the options for the dividers
	local zone = Textzone.new{width=self.c_desc.w, height=self.c_desc.h, text=string.toTString(fps_factor_explanation("resting"))}
	self.list[#(self.list)+1] = { zone=zone, name=string.toTString"#FIREBRICK##{bold}#Resting FPS Divider#WHITE##{normal}#", status=function(item)
			return tostring(config.settings.display_fps_resting_div)
		end, fct=function(item)
			local fps_factors = factors(config.settings.display_fps)
			local factor_dialog_height = 100 + (#fps_factors * 20)
			Dialog:listPopup("Resting FPS Divider", "Select divider", fps_factors, 200, factor_dialog_height, function(sel)
				if not sel then return end
				game:saveSettings("display_fps_resting_div", ("display_fps_resting_div = %d\n"):format(tonumber(sel.val)))
				config.settings.display_fps_resting_div = tonumber(sel.val)
				self.c_list:drawItem(item)
			end)
		end,}
		
	local zone = Textzone.new{width=self.c_desc.w, height=self.c_desc.h, text=string.toTString(fps_factor_explanation("running/exploring"))}
	self.list[#(self.list)+1] = { zone=zone, name=string.toTString"#FIREBRICK##{bold}#Run/Explore FPS Divider#WHITE##{normal}#", status=function(item)
			return tostring(config.settings.display_fps_running_div)
		end, fct=function(item)
			local fps_factors = factors(config.settings.display_fps)
			local factor_dialog_height = 100 + (#fps_factors * 20)
			Dialog:listPopup("Run/Explore FPS Divider", "Select divider", fps_factors, 200, factor_dialog_height, function(sel)
				if not sel then return end
				game:saveSettings("display_fps_running_div", ("display_fps_running_div = %d\n"):format(tonumber(sel.val)))
				config.settings.display_fps_running_div = tonumber(sel.val)
				self.c_list:drawItem(item)
				self:setupUI()
			end)
		end,}

end