-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011, 2012, 2013 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

local _M = loadPrevious(...)

local base_setupCommands = _M.setupCommands

--[[
--	Unfortunately, rest_before_explore prevents simply being able to 
--   superload RUN_AUTO, since resting will trigger a FPS drop->restore.
--
--  (i.e, dropFps -> base_RUN_AUTO -> restoreFPS won't work)
--
--  Also, setupCommands runs AFTER the ToME:run hook, so setupCommands must
--   be superloaded.
--]]
function _M:setupCommands()
	-- setup all the standard commands
	base_setupCommands(self)

	-- Define the new autoexplore function
	local fasterAutoExplore = function()
		local ae = function() if self.level and self.zone then
			local seen = {}
			-- Check for visible monsters.  Only see LOS actors, so telepathy wont prevent it
			core.fov.calc_circle(self.player.x, self.player.y, self.level.map.w, self.level.map.h, self.player.sight or 10,
				function(_, x, y) return self.level.map:opaque(x, y) end,
				function(_, x, y)
					local actor = self.level.map(x, y, self.level.map.ACTOR)
					if actor and actor ~= self.player and self.player:reactionToward(actor) < 0 and
						self.player:canSee(actor) and self.level.map.seens(x, y) then seen[#seen + 1] = {x=x, y=y, actor=actor} end
				end, nil)
			if self.zone.no_autoexplore or self.level.no_autoexplore then
				self.log("You may not auto-explore this level.")
			elseif #seen > 0 then
				local dir = game.level.map:compassDirection(seen[1].x - self.player.x, seen[1].y - self.player.y)
				self.log("You may not auto-explore with enemies in sight (%s to the %s%s)!", seen[1].actor.name, dir, self.level.map:isOnScreen(seen[1].x, seen[1].y) and "" or " - offscreen")
				for _, node in ipairs(seen) do
					node.actor:addParticles(engine.Particles.new("notice_enemy", 1))
				end
			else
				-- reduce FPS while running (for quicker exploration!)
				core.game.setFPS(config.settings.display_fps / config.settings.display_fps_running_div)

				if not self.player:autoExplore() then
					-- restore FPS (runStop was not called!)
					core.game.setFPS(config.settings.display_fps);
					self.log("There is nowhere left to explore.")
					
				end

			end
		end end

		if config.settings.tome.rest_before_explore then
			local ok = false
			self.player:restInit(nil, nil, nil, function() ok = self.player.resting.rested_fully end, function() if ok then self:onTickEnd(ae) self.tick_loopback = true end end)
		else
			ae()
		end
	end
	
	-- Overwrite the old keybind
	self.key:addBinds({RUN_AUTO = fasterAutoExplore})
	
end